@@ -22,27 +22,44 @@
 
 // Based on github.com/keplerproject/lua-compat-5.3
 
+use std::ffi::CStr;
 use std::mem;
 use std::os::raw::{c_char, c_int, c_void};
 use std::ptr;
 
 use super::lauxlib::{
-    luaL_Reg, luaL_callmeta, luaL_checkstack, luaL_checktype, luaL_error, luaL_getmetafield_51,
-    luaL_getmetatable, luaL_loadbuffer, luaL_newmetatable_51,
+    luaL_callmeta, luaL_error, luaL_getmetafield_old, luaL_loadbuffer, luaL_newmetatable_old,
 };
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
+use super::lauxlib::{luaL_Reg, luaL_checktype, luaL_getmetatable};
+
+#[cfg(feature = "lua52")]
+use super::lauxlib::{luaL_checkstack, luaL_getsubtable};
+
 use super::lua::{
-    self, lua_CFunction, lua_Debug, lua_Integer, lua_Number, lua_State, lua_call, lua_concat,
-    lua_createtable, lua_equal, lua_error, lua_getfenv, lua_getfield_51, lua_getinfo,
-    lua_getmetatable, lua_getstack, lua_gettable_51, lua_gettop, lua_insert, lua_isnumber,
-    lua_isstring, lua_istable, lua_lessthan, lua_newtable, lua_newuserdata, lua_next, lua_objlen,
-    lua_pop, lua_pushboolean, lua_pushcclosure, lua_pushcfunction, lua_pushfstring,
-    lua_pushinteger, lua_pushlightuserdata, lua_pushliteral, lua_pushlstring_51, lua_pushnil,
-    lua_pushnumber, lua_pushstring_51, lua_pushthread, lua_pushvalue, lua_rawequal, lua_rawget_51,
-    lua_rawgeti_51, lua_rawset, lua_remove, lua_replace, lua_resume_51, lua_setfenv, lua_setfield,
-    lua_setglobal, lua_setmetatable, lua_settable, lua_settop, lua_toboolean, lua_tointeger,
-    lua_tolstring, lua_tonumber, lua_topointer, lua_tostring, lua_touserdata, lua_type,
-    lua_typename,
+    self, lua_CFunction, lua_Debug, lua_Integer, lua_Number, lua_State, lua_Writer, lua_call,
+    lua_createtable, lua_dump_old, lua_error, lua_getfield_old, lua_getstack, lua_gettable_old,
+    lua_gettop, lua_insert, lua_isstring, lua_istable, lua_newuserdata, lua_pop, lua_pushboolean,
+    lua_pushcfunction, lua_pushfstring, lua_pushinteger, lua_pushliteral, lua_pushlstring_old,
+    lua_pushnumber, lua_pushthread, lua_pushvalue, lua_rawget_old, lua_rawgeti_old, lua_rawset,
+    lua_replace, lua_setfield, lua_setglobal, lua_setmetatable, lua_settable, lua_toboolean,
+    lua_tointeger, lua_tolstring, lua_tonumber, lua_topointer, lua_tostring, lua_touserdata,
+    lua_type, lua_typename,
+};
+
+#[cfg(any(feature = "lua51", feature = "luajit"))]
+use super::lua::{
+    lua_checkstack, lua_concat, lua_equal, lua_getfenv, lua_getinfo, lua_getmetatable,
+    lua_isnumber, lua_lessthan, lua_newtable, lua_next, lua_objlen, lua_pushcclosure,
+    lua_pushlightuserdata, lua_pushnil, lua_pushstring_old, lua_rawequal, lua_remove,
+    lua_resume_old, lua_setfenv, lua_settop, LUA_OPADD, LUA_OPUNM,
+};
+
+#[cfg(feature = "lua52")]
+use super::lua::{
+    lua_absindex, lua_getglobal_old, lua_getuservalue_old, lua_pushstring, lua_rawgetp_old,
+    lua_rawsetp, lua_tonumberx,
 };
 
 unsafe fn compat53_reverse(L: *mut lua_State, mut a: c_int, mut b: c_int) {
@@ -109,6 +126,7 @@ unsafe fn compat53_checkmode(
     lua::LUA_OK
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 unsafe fn compat53_findfield(L: *mut lua_State, objidx: c_int, level: c_int) -> c_int {
     if level == 0 || lua_istable(L, -1) == 0 {
         return 0; // not found
@@ -137,6 +155,7 @@ unsafe fn compat53_findfield(L: *mut lua_State, objidx: c_int, level: c_int) ->
     return 0; // not found
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 unsafe fn compat53_pushglobalfuncname(L: *mut lua_State, ar: *mut lua_Debug) -> c_int {
     let top = lua_gettop(L);
     lua_getinfo(L, cstr!("f"), ar); // push function
@@ -151,6 +170,7 @@ unsafe fn compat53_pushglobalfuncname(L: *mut lua_State, ar: *mut lua_Debug) ->
     }
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 unsafe fn compat53_pushfuncname(L: *mut lua_State, ar: *mut lua_Debug) {
     if *(*ar).namewhat != b'\0' as c_char {
         // is there a name?
@@ -199,11 +219,13 @@ unsafe fn compat53_call_lua(L: *mut lua_State, code: &str, nargs: c_int, nret: c
 // lua ported functions
 //
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 #[inline(always)]
 pub fn lua_upvalueindex(i: c_int) -> c_int {
     lua::LUA_GLOBALSINDEX - i
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_absindex(L: *mut lua_State, mut idx: c_int) -> c_int {
     if idx < 0 && idx > lua::LUA_REGISTRYINDEX {
         idx += lua_gettop(L) + 1;
@@ -211,6 +233,33 @@ pub unsafe fn lua_absindex(L: *mut lua_State, mut idx: c_int) -> c_int {
     idx
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
+static COMPAT53_ARITH_CODE: &'static str = r#"
+local op,a,b=...
+if op == 0 then return a+b
+elseif op == 1 then return a-b
+elseif op == 2 then return a*b
+elseif op == 3 then return a/b
+elseif op == 4 then return a%b
+elseif op == 5 then return a^b
+elseif op == 6 then return -a
+end
+"#;
+
+#[cfg(any(feature = "lua51", feature = "luajit"))]
+pub unsafe fn lua_arith(L: *mut lua_State, op: c_int) {
+    if op < LUA_OPADD || op > LUA_OPUNM {
+        luaL_error(L, cstr!("invalid 'op' argument for lua_arith"));
+    }
+    luaL_checkstack(L, 5, cstr!("not enough stack slots"));
+    if op == LUA_OPUNM {
+        lua_pushvalue(L, -1);
+    }
+    lua_pushnumber(L, op as lua_Number);
+    lua_insert(L, -3);
+    compat53_call_lua(L, COMPAT53_ARITH_CODE, 3, 1);
+}
+
 pub unsafe fn lua_rotate(L: *mut lua_State, mut idx: c_int, mut n: c_int) {
     idx = lua_absindex(L, idx);
     let n_elems = lua_gettop(L) - idx + 1;
@@ -226,6 +275,7 @@ pub unsafe fn lua_rotate(L: *mut lua_State, mut idx: c_int, mut n: c_int) {
     }
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_copy(L: *mut lua_State, fromidx: c_int, toidx: c_int) {
     let abs_to = lua_absindex(L, toidx);
     luaL_checkstack(L, 1, cstr!("not enough stack slots"));
@@ -244,6 +294,7 @@ pub unsafe fn lua_isinteger(L: *mut lua_State, idx: c_int) -> c_int {
     return 0;
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_tonumberx(L: *mut lua_State, i: c_int, isnum: *mut c_int) -> lua_Number {
     let n = lua_tonumber(L, i);
     if isnum != ptr::null_mut() {
@@ -256,6 +307,8 @@ pub unsafe fn lua_tonumberx(L: *mut lua_State, i: c_int, isnum: *mut c_int) -> l
     return n;
 }
 
+// Implemented for Lua 5.2 as well
+// See https://github.com/keplerproject/lua-compat-5.3/issues/40
 pub unsafe fn lua_tointegerx(L: *mut lua_State, i: c_int, isnum: *mut c_int) -> lua_Integer {
     let mut ok = 0;
     let n = lua_tonumberx(L, i, &mut ok);
@@ -273,10 +326,12 @@ pub unsafe fn lua_tointegerx(L: *mut lua_State, i: c_int, isnum: *mut c_int) ->
     return 0;
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_rawlen(L: *mut lua_State, idx: c_int) -> usize {
     lua_objlen(L, idx)
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_compare(L: *mut lua_State, mut idx1: c_int, mut idx2: c_int, op: c_int) -> c_int {
     match op {
         lua::LUA_OPEQ => lua_equal(L, idx1, idx2),
@@ -296,27 +351,44 @@ pub unsafe fn lua_compare(L: *mut lua_State, mut idx1: c_int, mut idx2: c_int, o
     }
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_pushlstring(L: *mut lua_State, s: *const c_char, l: usize) -> *const c_char {
     if l == 0 {
-        lua_pushlstring_51(L, cstr!(""), 0);
+        lua_pushlstring_old(L, cstr!(""), 0);
     } else {
-        lua_pushlstring_51(L, s, l);
+        lua_pushlstring_old(L, s, l);
     }
     lua_tostring(L, -1)
 }
 
+#[cfg(feature = "lua52")]
+pub unsafe fn lua_pushlstring(L: *mut lua_State, s: *const c_char, l: usize) -> *const c_char {
+    if l == 0 {
+        lua_pushlstring_old(L, cstr!(""), 0)
+    } else {
+        lua_pushlstring_old(L, s, l)
+    }
+}
+
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_pushstring(L: *mut lua_State, s: *const c_char) -> *const c_char {
-    lua_pushstring_51(L, s);
+    lua_pushstring_old(L, s);
     lua_tostring(L, -1)
 }
 
+#[cfg(feature = "lua52")]
+pub unsafe fn lua_getglobal(L: *mut lua_State, var: *const c_char) -> c_int {
+    lua_getglobal_old(L, var);
+    lua_type(L, -1)
+}
+
 pub unsafe fn lua_gettable(L: *mut lua_State, idx: c_int) -> c_int {
-    lua_gettable_51(L, idx);
+    lua_gettable_old(L, idx);
     lua_type(L, -1)
 }
 
 pub unsafe fn lua_getfield(L: *mut lua_State, idx: c_int, k: *const c_char) -> c_int {
-    lua_getfield_51(L, idx, k);
+    lua_getfield_old(L, idx, k);
     lua_type(L, -1)
 }
 
@@ -329,16 +401,17 @@ pub unsafe fn lua_geti(L: *mut lua_State, mut idx: c_int, n: lua_Integer) -> c_i
 
 // A new version which returns c_int
 pub unsafe fn lua_rawget(L: *mut lua_State, idx: c_int) -> c_int {
-    lua_rawget_51(L, idx);
+    lua_rawget_old(L, idx);
     lua_type(L, -1)
 }
 
 // A new version which returns c_int
 pub unsafe fn lua_rawgeti(L: *mut lua_State, idx: c_int, n: lua_Integer) -> c_int {
-    lua_rawgeti_51(L, idx, n);
+    lua_rawgeti_old(L, idx, n);
     lua_type(L, -1)
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_rawgetp(L: *mut lua_State, idx: c_int, p: *const c_void) -> c_int {
     let abs_i = lua_absindex(L, idx);
     lua_pushlightuserdata(L, p as *mut c_void);
@@ -346,11 +419,24 @@ pub unsafe fn lua_rawgetp(L: *mut lua_State, idx: c_int, p: *const c_void) -> c_
     lua_type(L, -1)
 }
 
+#[cfg(feature = "lua52")]
+pub unsafe fn lua_rawgetp(L: *mut lua_State, idx: c_int, p: *const c_void) -> c_int {
+    lua_rawgetp_old(L, idx, p);
+    lua_type(L, -1)
+}
+
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_getuservalue(L: *mut lua_State, idx: c_int) -> c_int {
     lua_getfenv(L, idx);
     lua_type(L, -1)
 }
 
+#[cfg(feature = "lua52")]
+pub unsafe fn lua_getuservalue(L: *mut lua_State, idx: c_int) -> c_int {
+    lua_getuservalue_old(L, idx);
+    lua_type(L, -1)
+}
+
 pub unsafe fn lua_seti(L: *mut lua_State, mut idx: c_int, n: lua_Integer) {
     luaL_checkstack(L, 1, cstr!("not enough stack slots available"));
     idx = lua_absindex(L, idx);
@@ -359,6 +445,7 @@ pub unsafe fn lua_seti(L: *mut lua_State, mut idx: c_int, n: lua_Integer) {
     lua_settable(L, idx);
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_rawsetp(L: *mut lua_State, idx: c_int, p: *const c_void) {
     let abs_i = lua_absindex(L, idx);
     luaL_checkstack(L, 1, cstr!("not enough stack slots"));
@@ -367,15 +454,27 @@ pub unsafe fn lua_rawsetp(L: *mut lua_State, idx: c_int, p: *const c_void) {
     lua_rawset(L, abs_i);
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_setuservalue(L: *mut lua_State, idx: c_int) {
     luaL_checktype(L, -1, lua::LUA_TTABLE);
     lua_setfenv(L, idx);
 }
 
-pub unsafe fn lua_resume(L: *mut lua_State, _from: *mut lua_State, narg: c_int) -> c_int {
-    lua_resume_51(L, narg)
+pub unsafe fn lua_dump(
+    L: *mut lua_State,
+    writer: lua_Writer,
+    data: *mut c_void,
+    _strip: c_int,
+) -> c_int {
+    lua_dump_old(L, writer, data)
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
+pub unsafe fn lua_resume(L: *mut lua_State, _from: *mut lua_State, narg: c_int) -> c_int {
+    lua_resume_old(L, narg)
+}
+
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn lua_len(L: *mut lua_State, idx: c_int) {
     match lua_type(L, idx) {
         lua::LUA_TSTRING => {
@@ -397,7 +496,18 @@ pub unsafe fn lua_len(L: *mut lua_State, idx: c_int) {
     }
 }
 
-// TODO: lua_stringtonumber
+pub unsafe fn lua_stringtonumber(L: *mut lua_State, s: *const c_char) -> usize {
+    use std::str::FromStr;
+
+    let cs = CStr::from_ptr(s);
+    if let Ok(rs) = cs.to_str() {
+        if let Ok(n) = f64::from_str(rs.trim()) {
+            lua_pushnumber(L, n as lua_Number);
+            return cs.to_bytes_with_nul().len();
+        }
+    }
+    0
+}
 
 pub unsafe fn lua_getextraspace(L: *mut lua_State) -> *mut c_void {
     use super::glue::LUA_EXTRASPACE;
@@ -452,6 +562,7 @@ pub unsafe fn lua_getextraspace(L: *mut lua_State) -> *mut c_void {
     return _ptr;
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 #[inline(always)]
 pub unsafe fn lua_pushglobaltable(L: *mut lua_State) {
     lua_pushvalue(L, lua::LUA_GLOBALSINDEX);
@@ -461,12 +572,24 @@ pub unsafe fn lua_pushglobaltable(L: *mut lua_State) {
 // lauxlib ported functions
 //
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
+pub unsafe fn luaL_checkstack(L: *mut lua_State, sz: c_int, msg: *const c_char) {
+    if lua_checkstack(L, sz + lua::LUA_MINSTACK) == 0 {
+        if msg != ptr::null() {
+            luaL_error(L, cstr!("stack overflow (%s)"), msg);
+        } else {
+            lua_pushliteral(L, "stack overflow");
+            lua_error(L);
+        }
+    }
+}
+
 pub unsafe fn luaL_checkversion(_L: *mut lua_State) {
     // Void
 }
 
 pub unsafe fn luaL_getmetafield(L: *mut lua_State, obj: c_int, e: *const c_char) -> c_int {
-    if luaL_getmetafield_51(L, obj, e) != 0 {
+    if luaL_getmetafield_old(L, obj, e) != 0 {
         lua_type(L, -1)
     } else {
         lua::LUA_TNIL
@@ -474,7 +597,7 @@ pub unsafe fn luaL_getmetafield(L: *mut lua_State, obj: c_int, e: *const c_char)
 }
 
 pub unsafe fn luaL_newmetatable(L: *mut lua_State, tname: *const c_char) -> c_int {
-    if luaL_newmetatable_51(L, tname) != 0 {
+    if luaL_newmetatable_old(L, tname) != 0 {
         lua_pushstring(L, tname);
         lua_setfield(L, -2, cstr!("__name"));
         1
@@ -483,6 +606,7 @@ pub unsafe fn luaL_newmetatable(L: *mut lua_State, tname: *const c_char) -> c_in
     }
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn luaL_loadbufferx(
     L: *mut lua_State,
     buff: *const c_char,
@@ -501,6 +625,7 @@ pub unsafe fn luaL_loadbufferx(
     luaL_loadbuffer(L, buff, sz, name)
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn luaL_len(L: *mut lua_State, idx: c_int) -> lua_Integer {
     let mut isnum = 0;
     luaL_checkstack(L, 1, cstr!("not enough stack slots"));
@@ -513,6 +638,7 @@ pub unsafe fn luaL_len(L: *mut lua_State, idx: c_int) -> lua_Integer {
     res
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn luaL_traceback(
     L: *mut lua_State,
     L1: *mut lua_State,
@@ -590,12 +716,14 @@ pub unsafe fn luaL_tolstring(L: *mut lua_State, idx: c_int, len: *mut usize) ->
     lua_tolstring(L, -1, len)
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn luaL_setmetatable(L: *mut lua_State, tname: *const c_char) {
     luaL_checkstack(L, 1, cstr!("not enough stack slots"));
     luaL_getmetatable(L, tname);
     lua_setmetatable(L, -2);
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn luaL_testudata(L: *mut lua_State, i: c_int, tname: *const c_char) -> *mut c_void {
     let mut p = lua_touserdata(L, i);
     luaL_checkstack(L, 2, cstr!("not enough stack slots"));
@@ -612,6 +740,7 @@ pub unsafe fn luaL_testudata(L: *mut lua_State, i: c_int, tname: *const c_char)
     return p;
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn luaL_setfuncs(L: *mut lua_State, mut l: *const luaL_Reg, nup: c_int) {
     luaL_checkstack(L, nup + 1, cstr!("too many upvalues"));
     while (*l).name != ptr::null() {
@@ -628,6 +757,7 @@ pub unsafe fn luaL_setfuncs(L: *mut lua_State, mut l: *const luaL_Reg, nup: c_in
     lua_pop(L, nup); // remove upvalues
 }
 
+#[cfg(any(feature = "lua51", feature = "luajit"))]
 pub unsafe fn luaL_getsubtable(L: *mut lua_State, idx: c_int, fname: *const c_char) -> c_int {
     let abs_i = lua_absindex(L, idx);
     luaL_checkstack(L, 3, cstr!("not enough stack slots"));
